/*
 * Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2.0,
 * as published by the Free Software Foundation.
 *
 * This program is designed to work with certain software (including
 * but not limited to OpenSSL) that is licensed under separate terms, as
 * designated in a particular file or component or in included license
 * documentation.  The authors of MySQL hereby grant you an additional
 * permission to link the program and your derivative works with the
 * separately licensed software that they have either included with
 * the program or referenced in the documentation.
 * This program is distributed in the hope that it will be useful,  but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 * the GNU General Public License, version 2.0, for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#pragma once

#include "common.h"

namespace mga {
  enum class Property : size_t {
    Text,
    Title,
    Description,
    Enabled,
    CanExpand,
    Expanded,
    CanFocus,
    Focused,
    CheckState,
    Value,
    MaxValue,
    MinValue,
    ActiveTab,
    Active,
    Editable,
    SelectedIndexes,
    ReadOnly,
    IsSecure,
    CaretPosition,
    SelectedText,
    SelectionRange,
    CharacterCount,
    Horizontal,
    Date,
    SelectedIndex,
    Selected,
    Help,
    Range,
    Position,
    Shown,
    Bounds,
    ScrollLeft,
    ScrollRight,
    ScrollUp,
    ScrollDown,
    Increment,
    Decrement,
    StepUp,
    StepDown,

    Rows,
    Columns,
    Pages,
    CloseButton,
    Entries,
    Header,
    HorizontalScrollbar,
    VerticalScrollbar,
    Screen,

    Press,
    Activate,
    Show,
    ShowContextMenu,
    TakeScreenShot,
    GetBounds,
    InsertText,

    PropertyCount // Sentinel
  };

  std::string propertyToString(const Property property);
  Property propertyFromString(const std::string text);
}
